#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "queue.h"

void
init_queue(queue *q, size_t mem_sz)
{
        q->queue_sz = 0;
        q->mem_sz = mem_sz;
        q->head = q->tail = NULL;
}

int
enqueue(queue *q, const void *data)
{
        node *new_node = (node *) malloc(sizeof(node));

        if (new_node == NULL)
            return -1;

        new_node->data = malloc(q->mem_sz);

        if (new_node->data == NULL) {
            free(new_node);
            return -1;
        }

        new_node->next = NULL;

        memcpy(new_node->data, data, q->mem_sz);

        if (q->queue_sz == 0)
                q->head = q->tail = new_node;
        else {
                q->tail->next = new_node;
                q->tail = new_node;
        }

        q->queue_sz++;
        
        return 0;
}

void
dequeue(queue *q, void *data)
{
        if (q->queue_sz > 0) {
                node *temp = q->head;
                memcpy(data, temp->data, q->mem_sz);

                if (q->queue_sz > 1)
                    q->head = q->head->next;
                else {
                    q->head = NULL;
                    q->tail = NULL;
                }

                q->queue_sz--;
                free(temp->data);
                free(temp);
        }
}

int
check_queue_sz(queue *q)
{
        return q->queue_sz;
}

void
free_queue(queue *q)
{
        node        *temp;

        while (q->queue_sz > 0) {
                temp = q->head;
                q->head = temp->next;
                free(temp->data);
                free(temp);
                q->queue_sz--;
        }

        q->head = q->tail = NULL;
}
