#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "queue.h"

#define _THREADS_ 2

void            *thread_func(void *);

pthread_mutex_t t_mutex;

void *
thread_func(void *vargs)
{
        queue               *sq;
        pthread_t            tid;
        int                  val;

	if (pthread_mutex_init(&t_mutex, NULL) != 0) {
		perror("pthread_mutex_init failed");
		exit(1);
	}
	
        sq = (queue *) vargs;
        tid = pthread_self();

	/*
	 * Lock queue access, pop element, unlock;
	 * repeat between two threads till empty
	 */
	pthread_mutex_lock(&t_mutex);
        while(check_queue_sz(sq) > 0) {
                dequeue(sq, &val);
                printf("thread id: %u  |  processing %d\n", (uint) tid, val);
        	sleep(1);
        }
	if (!(pthread_mutex_trylock(&t_mutex)))
		pthread_mutex_unlock(&t_mutex);

        pthread_exit(NULL);
}


int
main(int argc, char **argv)
{
        queue               q;
        pthread_t           tid[_THREADS_] = {0};
        int                 task;
	
	/* Load queue with ten "tasks" to be processed by two threads */
        init_queue(&q, sizeof(int));

        for(task = 0; task < 10; ++task) {
                enqueue(&q, &task);
                printf("task %d added to queue\n", task);
        }
        printf("\n");

        for (int i = 0; i < _THREADS_; ++i) {
                if(pthread_create(&tid[i], NULL, thread_func, (void *) &q) != 0 ) {
                        perror("pthread_create failed");
                        exit(1);
                }
                printf("thread %u starting...\n", (uint) tid[i]);
        }
        printf("\n");

        for (int i = 0; i < _THREADS_; ++i) {
                if (pthread_join(tid[i], NULL) != 0)
                        perror("pthread_join failed");
                printf("thread %u exiting...\n", (uint) tid[i]);
        }
        printf("\n");

        pthread_mutex_destroy(&t_mutex);

        return 0;
}
